@@ -86,9 +86,11 @@ def getIpAddress(hostname, workaround127=False, ipVersion=None):
             ip=getInterfaceAddress("4.2.2.2")
         return ip
     try:
-        return getaddr(Pyro4.config.PREFER_IP_VERSION) if ipVersion == None else getaddr(ipVersion)
+        if hostname and ':' in hostname and ipVersion is None:
+            ipVersion = 0
+        return getaddr(Pyro4.config.PREFER_IP_VERSION) if ipVersion is None else getaddr(ipVersion)
     except socket.gaierror:
-        if ipVersion == 6 or (ipVersion == None and Pyro4.config.PREFER_IP_VERSION == 6):
+        if ipVersion == 6 or (ipVersion is None and Pyro4.config.PREFER_IP_VERSION == 6):
             # try a (inefficient, but hey) workaround to obtain the ipv6 address:
             # attempt to connect to one of a few ipv6-servers (google's public dns servers),
             # and obtain the connected socket's address. (This will only work with an active internet connection)
@@ -107,10 +109,11 @@ def getInterfaceAddress(ip_address):
     """tries to find the ip address of the interface that connects to the given host's address"""
     family = socket.AF_INET if getIpVersion(ip_address)==4 else socket.AF_INET6
     sock = socket.socket(family, socket.SOCK_DGRAM)
-    sock.connect((ip_address, 53))   # 53=dns
-    ip = sock.getsockname()[0]
-    sock.close()
-    return ip
+    try:
+        sock.connect((ip_address, 53))   # 53=dns
+        return sock.getsockname()[0]
+    finally:
+        sock.close()
 
 
 def __nextRetrydelay(delay):
@@ -201,15 +204,14 @@ def sendData(sock, data):
     """
     if sock.gettimeout() is None:
         # socket is in blocking mode, we can use sendall normally.
-        while True:
-            try:
-                sock.sendall(data)
-                return
-            except socket.timeout:
-                raise TimeoutError("sending: timeout")
-            except socket.error:
-                x=sys.exc_info()[1]
-                raise ConnectionClosedError("sending: connection lost: "+str(x))
+        try:
+            sock.sendall(data)
+            return
+        except socket.timeout:
+            raise TimeoutError("sending: timeout")
+        except socket.error:
+            x=sys.exc_info()[1]
+            raise ConnectionClosedError("sending: connection lost: "+str(x))
     else:
         # Socket is in non-blocking mode, use regular send loop.
         retrydelay=0.0
@@ -230,7 +232,7 @@ def sendData(sock, data):
 
 _GLOBAL_DEFAULT_TIMEOUT=object()
 
-def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeout=_GLOBAL_DEFAULT_TIMEOUT, noinherit=False, ipv6=False):
+def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeout=_GLOBAL_DEFAULT_TIMEOUT, noinherit=False, ipv6=False, nodelay=False):
     """
     Create a socket. Default socket options are keepalive and IPv4 family.
     If 'bind' or 'connect' is a string, it is assumed a Unix domain socket is requested.
@@ -240,7 +242,7 @@ def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeo
     """
     if bind and connect:
         raise ValueError("bind and connect cannot both be specified at the same time")
-    forceIPv6=ipv6 or (ipv6 == None and Pyro4.config.PREFER_IP_VERSION == 6)
+    forceIPv6=ipv6 or (ipv6 is None and Pyro4.config.PREFER_IP_VERSION == 6)
     if type(bind) is str or type(connect) is str:
         family=socket.AF_UNIX
     elif not bind and not connect:
@@ -276,6 +278,8 @@ def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeo
     else:
         raise ValueError("unknown bind or connect format.")
     sock=socket.socket(family, socket.SOCK_STREAM)
+    if nodelay:
+        setNoDelay(sock)
     if reuseaddr:
         setReuseAddr(sock)
     if noinherit:
@@ -326,7 +330,7 @@ def createBroadcastSocket(bind=None, reuseaddr=False, timeout=_GLOBAL_DEFAULT_TI
     Set ipv6=True to create an IPv6 socket rather than IPv4.
     Set ipv6=None to use the PREFER_IP_VERSION config setting.
     """
-    forceIPv6=ipv6 or (ipv6 == None and Pyro4.config.PREFER_IP_VERSION == 6)
+    forceIPv6=ipv6 or (ipv6 is None and Pyro4.config.PREFER_IP_VERSION == 6)
     if not bind:
         family=socket.AF_INET6 if forceIPv6 else socket.AF_INET
     elif type(bind) is tuple:
@@ -352,9 +356,7 @@ def createBroadcastSocket(bind=None, reuseaddr=False, timeout=_GLOBAL_DEFAULT_TI
     if timeout is None:
         sock.settimeout(None)
     else:
-        if timeout is not _GLOBAL_DEFAULT_TIMEOUT and (not bind or os.name!="java"):
-            # only set timeout on the udp socket in this case, because Jython
-            # has a problem with timeouts on udp sockets, see http://bugs.jython.org/issue1018
+        if timeout is not _GLOBAL_DEFAULT_TIMEOUT:
             sock.settimeout(timeout)
     if bind:
         host = bind[0] or ""
@@ -379,6 +381,14 @@ def setReuseAddr(sock):
         pass
 
 
+def setNoDelay(sock):
+    """sets the TCP_NODELAY option on the socket (to disable Nagle's algorithm), if possible."""
+    try:
+        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+    except Exception:
+        pass
+
+
 def setKeepalive(sock):
     """sets the SO_KEEPALIVE option on the socket, if possible."""
     try:
